@@ -84,6 +84,6 @@ sc_arc_base <- function(path_link_vertex, node) {
   arcs0[["arc_"]] <- sc_uid(nlevels(f))[f]
   arcs0 <- dplyr::select(arcs0, .data$object_, .data$arc_, .data$vertex_)
   path_link_vertex <- path_link_vertex %>% inner_join(noded_path, "path_")
-  paths <- split(path_link_vertex, path_link_vertex[["path_"]])
+  paths <- split(path_link_vertex, path_link_vertex[["path_"]])[unique(path_link_vertex[["path_"]])]
   bind_rows(arcs0, bind_rows(lapply(paths, function(x) find_arc(x, node[["vertex_"]]))))
 }
